#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <vga.h>

#include "lib/safem.h"
#include "lib/log.h"
#include "video.h"
#include <ggi/libggi.h>

/* MIN_PAGES should normally be 2, but set it to 1 to force direct video */

#define MAX_PAGES 3
#define MIN_PAGES 2

static int pagelen=0;
static char *pagev=NULL;

static int mywidth,myheight;

static ggi_visual_t vis;

void video_preinit(void) {

   ggiInit();
   vis=ggiOpen(NULL);
   ggiSetFocus(vis);

};

void init_video(int *_width,int *_height,int *_bpp) {

   int err;
   if(pagev!=NULL) safe_free(pagev);
   pagev=NULL;

   switch(*_bpp) {
    case(4):
      err=ggiSetGraphMode(vis,*_width,*_height,*_width,*_height,GT_32BIT);
      break;
    case(2):
      err=ggiSetGraphMode(vis,*_width,*_height,*_width,*_height,GT_16BIT);
      break;
    case(1):
      err=ggiSetGraphMode(vis,*_width,*_height,*_width,*_height,GT_8BIT);
      break;
    default:
      logfatal('V',"Bad BPP (%d) in init_video",*_bpp);
   };

   if (err) logfatal('V',"GGI:SetGraphMode failed.");

   mywidth =*_width =ggiGetInfo(vis)->fb.width;
   myheight=*_height=ggiGetInfo(vis)->fb.height;
   
   pagelen=(*_width)*(*_height)*(*_bpp);

   logprintf(LOG_INFO,'V',"init_video in %s mode pagelen=%d","copying",pagelen);
   pagev=safe_malloc(pagelen);
};

void reset_video(void) {
   if(pagev!=NULL) safe_free(pagev);
   pagev=NULL;
   ggiExit();
};

#define SHLPAL 8L
void video_setpal(unsigned char idx,
		  unsigned char red,
		  unsigned char green,
		  unsigned char blue) {
   ggi_color gcol;
   gcol.r=red   << SHLPAL;
   gcol.g=green << SHLPAL;
   gcol.b=blue  << SHLPAL;

   ggiSetPaletteVec(vis,idx,1,&gcol);

};

void *video_newframe(void) {
   return pagev;
};

void video_updateframe(void *v) {

	int y;
#if 0	/* Fixme - looks like this is broken ... */
	ggiPutBox(vis,0,0,mywidth,myheight,v);
#else
	for(y=0;y<myheight;y++)
		ggiPutHLine(vis,0,y,mywidth,(char *)v+mywidth*y);
#endif
};

void video_winstuff(const char *desc,int xdim,int ydim) {};


#include "input.h"

static PlayerInput inca;

void get_input(PlayerInput *in) {

   static int speed=1;
   int sym;
   ggi_event ev;
   struct timeval t={0,0};

   if (ggiEventPoll(vis,emKeyPress|emKeyRelease,&t))
   {
	ggiEventPoll(vis,emKeyPress|emKeyRelease,NULL);

	do {
		ggiEventRead(vis,&ev,emKeyboard);
	} while(ev.any.type != evKeyPress &&
	        ev.any.type != evKeyRelease );

	sym=ev.key.sym;
	switch(sym)
	{
	  case 62979: /* CrSr up */
	  case 61960: /* CrSr up */
	  case 63235: /* CrSr up */
	  	if (ev.any.type == evKeyRelease )
	  	  inca.forward=0;
	  	else
	  	  inca.forward+=speed;
	  	break;
	  case 62976: /* CrSr down */
	  case 62210: /* CrSr down */
	  	if (ev.any.type == evKeyRelease )
	  	  inca.forward=0;
	  	else
	  	  inca.forward-=speed;
	  	break;
	  case 62978: /* CrSr right */
	  case 62214: /* CrSr right */
	  	if (ev.any.type == evKeyRelease )
	  	  inca.rotate=0;
	  	else
	  	  inca.rotate-=speed;
	  	break;
	  case 62977: /* CrSr left */
	  case 62212: /* CrSr left */
	  	if (ev.any.type == evKeyRelease )
	  	  inca.rotate=0;
	  	else
	  	  inca.rotate+=speed;
	  	break;

	  case 61953: /* enter */
	  	if (ev.any.type == evKeyRelease )
	  	  inca.shoot=0;
	  	else
	  	  inca.shoot=1;
	  	break;
	  case 61472: /* space */
	  	if (ev.any.type == evKeyRelease )
	  	  inca.jump=0;
	  	else
	  	  inca.jump=1;
	  	break;
	  case 61467: /* esc */
	  	if (ev.any.type == evKeyRelease )
	  	  inca.quit=0;
	  	else
	  	  inca.quit=1;
	  	break;
	  case 61484:
	  	if (ev.any.type == evKeyRelease )
	  	  inca.sideways=0;
	  	else
	  	  inca.sideways+=speed;
	  	break;
	  case 61486:
	  	if (ev.any.type == evKeyRelease )
	  	  inca.sideways=0;
	  	else
	  	  inca.sideways-=speed;
	  	break;
	  case 62217:
	  	if (ev.any.type == evKeyRelease )
	  	  inca.lookup=0;
	  	else
	  	  inca.lookup+=1;
	  	break;
	  case 62211:
	  	if (ev.any.type == evKeyRelease )
	  	  inca.lookup=0;
	  	else
	  	  inca.lookup-=1;
	  	break;
	  case 63232:
	  	if (ev.any.type == evKeyRelease )
	  	  speed=1;
	  	else
	  	  speed=2;
	  	break;

	  default: printf("sym=%d\n",sym);break;
	}
   }
   memcpy(in,&inca,sizeof(PlayerInput)); 
#if 0

#define NK(x) if(keyboard_keypressed(SCANCODE_##x)) in->select[x]=1
      NK(0);NK(1);NK(2);NK(3);NK(4);NK(5);NK(6);NK(7);NK(8);NK(9);
#undef NK   

#endif

};

void init_input(void) { 
   memset(&inca,0,sizeof(PlayerInput)); 
};

void reset_input(void) { };

