
#ifndef WADIO_H
#define WADIO_H

/*#include "endian.h" See below */

void reset_wad(void);
void init_iwad(const char *fname);
void init_pwad(const char *fname);

typedef unsigned int LumpNum;

#define LUMP_WADNUM(x) (((LumpNum)x)>>16)
#define LUMP_DIRNUM(x) (((LumpNum)x)&0xffff)
#define LUMPNUM(w,d) ( (((LumpNum)w)<<16) | (d&0xffff) )

#define BAD_LUMPNUM ((LumpNum)0xffffffff)
#define LUMPNUM_OK(l) (l!=BAD_LUMPNUM)

int count_lumps_between(const char *after,const char *before);

LumpNum safe_lookup_lump(const char *name,const char *after,const char *before,int lvl);
#define lookup_lump(x,y,z) safe_lookup_lump(x,y,z,-1)
#define getlump(n) safe_lookup_lump(n,NULL,NULL,LOG_FATAL)
#define have_lump(n) LUMPNUM_OK(lookup_lump(n,NULL,NULL))

int get_lump_fd(LumpNum ln);
unsigned int get_lump_ofs(LumpNum ln);
unsigned int get_lump_len(LumpNum ln);
const void *get_lump_map(LumpNum ln); 
char *get_lump_name(LumpNum ln,char *buf); 

int get_num_wads(void);
int get_num_lumps(unsigned int wadnum);

const void *load_lump(LumpNum ln);
void *copy_lump(LumpNum ln);
void release_lump(LumpNum ln);
void free_lump(LumpNum ln);
/* free_lump is for when we don't think we'll be wanting this one again */
void free_all_lumps(void);
void free_dead_lumps(void);

/*

Too complicated to do this properly with WAD files, since the conversions 
need to be all over the place, and we don't know whether changes persist
over calls to free_lump() - that will vary with the way mmapping works 
and/or how lumps are cached.

#define WAD_ENDIANISE16(x) FROM_LE16(x)
#define WAD_ENDIANISE32(x) FROM_LE32(x)
#ifdef BIG_ENDIAN
#define WAD_WRONG_ENDIAN
#endif
*/

#endif
